// Copyright (C) 2019-2020 Zilliz. All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance
// with the License. You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software distributed under the License
// is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions and limitations under the License.

package queryservice

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestReplica_Release(t *testing.T) {
	replica := newMetaReplica()
	err := replica.addCollection(0, 1, nil)
	require.NoError(t, err)

	colls, err := replica.getCollections(0)
	assert.NoError(t, err)
	assert.Equal(t, 1, len(colls))

	err = replica.addPartition(0, 1, 100)
	assert.NoError(t, err)
	partitions, err := replica.getPartitions(0, 1)
	assert.NoError(t, err)
	assert.Equal(t, 1, len(partitions))

	err = replica.releasePartition(0, 1, 100)
	assert.NoError(t, err)
	partitions, err = replica.getPartitions(0, 1)
	assert.NoError(t, err)
	assert.Equal(t, 0, len(partitions))
	err = replica.releasePartition(0, 1, 100)
	assert.Error(t, err)

	err = replica.releaseCollection(0, 1)
	assert.NoError(t, err)
	colls, err = replica.getCollections(0)
	assert.NoError(t, err)
	assert.Equal(t, 0, len(colls))
	err = replica.releaseCollection(0, 1)
	assert.Error(t, err)
}
